use serde::{Deserialize, Serialize};
use serde_json::Value;

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamResponse {
    pub team: Team,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Team {
    pub id: String,
    pub uid: String,
    pub slug: String,
    pub location: String,
    pub name: String,
    pub nickname: String,
    pub abbreviation: String,
    pub display_name: String,
    pub short_display_name: String,
    pub color: String,
    pub alternate_color: String,
    pub is_active: bool,
    pub logos: Vec<Logo>,
    pub record: TeamRecord,
    pub groups: TeamGroups,
    pub links: Vec<TeamLink>,
    pub franchise: TeamFranchise,
    // #[serde(rename = "nextEvent")]
    pub next_event: Vec<NextEvent>,
    pub standing_summary: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Logo {
    pub href: String,
    pub width: i64,
    pub height: i64,
    pub alt: String,
    pub rel: Vec<String>,
    pub last_updated: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamRecord {
    pub items: Vec<Item>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Item {
    #[serde(rename = "type")]
    pub type_field: String,
    pub summary: String,
    pub stats: Vec<Stat>,
    pub description: Option<String>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Stat {
    pub name: String,
    pub value: f64,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamGroups {
    pub id: String,
    pub parent: Parent,
    pub is_conference: bool,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Parent {
    pub id: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamLink {
    pub language: String,
    pub rel: Vec<String>,
    pub href: String,
    pub text: String,
    pub short_text: String,
    pub is_external: bool,
    pub is_premium: bool,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamFranchise {
    #[serde(rename = "$ref")]
    pub ref_field: String,
    pub id: String,
    pub uid: String,
    pub slug: String,
    pub location: String,
    pub name: String,
    pub nickname: String,
    pub abbreviation: String,
    pub display_name: String,
    pub short_display_name: String,
    pub color: String,
    pub is_active: bool,
    pub venue: TeamVenue,
    pub team: Team2,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamVenue {
    #[serde(rename = "$ref")]
    pub ref_field: String,
    pub id: String,
    pub full_name: String,
    pub address: TeamAddress,
    pub grass: bool,
    pub indoor: bool,
    pub images: Vec<TeamImage>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamAddress {
    pub city: String,
    pub state: String,
    pub zip_code: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamImage {
    pub href: String,
    pub width: i64,
    pub height: i64,
    pub alt: String,
    pub rel: Vec<String>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Team2 {
    #[serde(rename = "$ref")]
    pub ref_field: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct NextEvent {
    pub id: String,
    pub date: String,
    pub name: String,
    pub short_name: String,
    pub season: TeamSeason,
    pub season_type: TeamSeasonType,
    pub week: TeamWeek,
    pub time_valid: bool,
    pub competitions: Vec<TeamCompetition>,
    pub links: Vec<TeamLink4>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamSeason {
    pub year: i64,
    pub display_name: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamSeasonType {
    pub id: String,
    #[serde(rename = "type")]
    pub type_field: i64,
    pub name: String,
    pub abbreviation: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamWeek {
    pub number: i64,
    pub text: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamCompetition {
    pub id: String,
    pub date: String,
    pub attendance: i64,
    #[serde(rename = "type")]
    pub type_field: TeamType,
    pub time_valid: bool,
    pub neutral_site: bool,
    pub boxscore_available: bool,
    pub tickets_available: bool,
    pub venue: TeamVenue2,
    pub competitors: Vec<TeamCompetitor>,
    pub notes: Vec<Value>,
    pub broadcasts: Vec<Broadcast>,
    pub tickets: Vec<Ticket>,
    pub status: TeamStatus,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamType {
    pub id: String,
    pub text: String,
    pub abbreviation: String,
    pub slug: String,
    #[serde(rename = "type")]
    pub type_field: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamVenue2 {
    pub full_name: String,
    pub address: TeamAddress2,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamAddress2 {
    pub city: String,
    pub state: String,
    pub zip_code: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamCompetitor {
    pub id: String,
    #[serde(rename = "type")]
    pub type_field: String,
    pub order: i64,
    pub home_away: String,
    pub team: Team3,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Team3 {
    pub id: String,
    pub location: String,
    pub nickname: String,
    pub abbreviation: String,
    pub display_name: String,
    pub short_display_name: String,
    pub logos: Vec<Logo2>,
    pub links: Vec<TeamLink2>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Logo2 {
    pub href: String,
    pub width: i64,
    pub height: i64,
    pub alt: String,
    pub rel: Vec<String>,
    pub last_updated: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamLink2 {
    pub rel: Vec<String>,
    pub href: String,
    pub text: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Broadcast {
    #[serde(rename = "type")]
    pub type_field: Type2,
    pub market: Market,
    pub media: Media,
    pub lang: String,
    pub region: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Type2 {
    pub id: String,
    pub short_name: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Market {
    pub id: String,
    #[serde(rename = "type")]
    pub type_field: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Media {
    pub short_name: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Ticket {
    pub id: String,
    pub summary: String,
    pub description: String,
    pub max_price: f64,
    pub starting_price: f64,
    pub number_available: i64,
    pub total_postings: i64,
    pub links: Vec<TeamLink3>,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamLink3 {
    pub rel: Vec<String>,
    pub href: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamStatus {
    pub clock: f64,
    pub display_clock: String,
    pub period: i64,
    #[serde(rename = "type")]
    pub type_field: TeamType3,
    #[serde(rename = "isTBDFlex")]
    pub is_tbdflex: bool,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamType3 {
    pub id: String,
    pub name: String,
    pub state: String,
    pub completed: bool,
    pub description: String,
    pub detail: String,
    pub short_detail: String,
}

#[derive(Default, Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct TeamLink4 {
    pub language: String,
    pub rel: Vec<String>,
    pub href: String,
    pub text: String,
    pub short_text: String,
    pub is_external: bool,
    pub is_premium: bool,
}
